@@ -34,7 +34,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
     let vertex = BuildGraph.Unique.value v in
     match vertex with
     | BuildGraph.SrcPkg id | BuildGraph.InstSet (id,_) ->
-      CudfAdd.inttovar U.univ id
+      CudfAdd.inttopkg U.univ id
 
   let string_of_vertex ?(noversion=false) v =
     let pkg = pkg_of_vertex v in
@@ -44,14 +44,14 @@ module Make (U : sig val univ : Cudf.universe end) = struct
     let v = BuildGraph.Unique.value v in
     match v with
     | BuildGraph.SrcPkg id -> begin
-        let pkg = CudfAdd.inttovar U.univ id in
+        let pkg = CudfAdd.inttopkg U.univ id in
         (`SrcPkg (BootstrapCommon.debsrctuple_of_cudfpkg pkg))
       end
     | BuildGraph.InstSet (id,iss) -> begin
-        let pkg = CudfAdd.inttovar U.univ id in
+        let pkg = CudfAdd.inttopkg U.univ id in
         let sl =
           List.map (fun pid ->
-              let pkg = CudfAdd.inttovar U.univ pid in
+              let pkg = CudfAdd.inttopkg U.univ pid in
               BootstrapCommon.debbintriplet_of_cudfpkg pkg
             ) (IntSet.elements iss)
         in
@@ -80,7 +80,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
         let vertex = BuildGraph.Unique.value v in
         match vertex with
         | BuildGraph.SrcPkg id ->
-          (CudfAdd.inttovar U.univ id)::acc
+          (CudfAdd.inttopkg U.univ id)::acc
         | _ -> acc
       ) g []
 
@@ -91,7 +91,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
       let vertex1 = BuildGraph.Unique.value v1 in
       match vertex1 with
         | BuildGraph.SrcPkg id -> begin
-            let p1 = CudfAdd.inttovar U.univ id in
+            let p1 = CudfAdd.inttopkg U.univ id in
             let droppable = Hashtbl.find_default reduced_deps_ht p1.Cudf.package StringSet.empty in
             if StringSet.is_empty droppable then ()
             else begin
@@ -99,7 +99,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
                 let vertex2 = BuildGraph.Unique.value v2 in
                 match vertex2 with
                   | BuildGraph.InstSet (id,_) ->
-                      let p2 = CudfAdd.inttovar U.univ id in
+                      let p2 = CudfAdd.inttopkg U.univ id in
                       if StringSet.mem p2.Cudf.package droppable then
                         G.remove_edge g v1 v2;
                   | _ -> failwith("impossible")
@@ -116,7 +116,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
       let vertex = BuildGraph.Unique.value v in
       match vertex with
         | BuildGraph.InstSet (id,_) ->
-            let pkg = CudfAdd.inttovar U.univ id in
+            let pkg = CudfAdd.inttopkg U.univ id in
             if StringSet.mem pkg.Cudf.package weak_deps_set then
               G.remove_vertex g v
         | _ -> ()
@@ -285,7 +285,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
     VertexSet.iter (fun v1 ->
       let vertex1 = BuildGraph.Unique.value v1 in
       let src = match vertex1 with
-        | BuildGraph.SrcPkg id -> CudfAdd.inttovar U.univ id
+        | BuildGraph.SrcPkg id -> CudfAdd.inttopkg U.univ id
         | _ -> failwith "impossible"
       in
       let droppable = Hashtbl.find reduced_deps_ht src.Cudf.package in
@@ -293,7 +293,7 @@ module Make (U : sig val univ : Cudf.universe end) = struct
         let vertex2 = BuildGraph.Unique.value v2 in
         match vertex2 with
           | BuildGraph.InstSet (id,_) ->
-              let pkg = CudfAdd.inttovar U.univ id in
+              let pkg = CudfAdd.inttopkg U.univ id in
               if StringSet.mem pkg.Cudf.package droppable then
                 G.remove_edge g v1 v2
           | _ -> failwith "impossible"
@@ -315,12 +315,12 @@ module Make (U : sig val univ : Cudf.universe end) = struct
           | (sv,{ BuildGraph.depend = BuildGraph.BuildDep },iv) -> begin
               let s = BuildGraph.Unique.value sv in
               let src = match s with
-                | BuildGraph.SrcPkg id -> CudfAdd.inttovar U.univ id
+                | BuildGraph.SrcPkg id -> CudfAdd.inttopkg U.univ id
                 | _ -> failwith "impossible"
               in
               let p = BuildGraph.Unique.value iv in
               let pkg = match p with
-                | BuildGraph.InstSet (id,_) -> CudfAdd.inttovar U.univ id
+                | BuildGraph.InstSet (id,_) -> CudfAdd.inttopkg U.univ id
                 | _ -> failwith "impossible"
               in
               (* don't do anything if the source package doesnt have a build
@@ -394,12 +394,12 @@ module Make (U : sig val univ : Cudf.universe end) = struct
         | [(_,{ BuildGraph.depend = BuildGraph.BuildsFrom _ },_);(sv,{ BuildGraph.depend = BuildGraph.BuildDep },iv)] ->
             let s = BuildGraph.Unique.value sv in
             let src = match s with
-              | BuildGraph.SrcPkg id -> CudfAdd.inttovar U.univ id
+              | BuildGraph.SrcPkg id -> CudfAdd.inttopkg U.univ id
               | _ -> failwith "impossible"
             in
             let p = BuildGraph.Unique.value iv in
             let pkg = match p with
-              | BuildGraph.InstSet (id,_) -> CudfAdd.inttovar U.univ id
+              | BuildGraph.InstSet (id,_) -> CudfAdd.inttopkg U.univ id
               | _ -> failwith "impossible"
             in
             let droppable = try
@@ -441,17 +441,17 @@ module Make (U : sig val univ : Cudf.universe end) = struct
       let vertex2 = BuildGraph.Unique.value v2 in
       match (vertex1,label,vertex2) with
         | (BuildGraph.SrcPkg sid, {BuildGraph.depend = BuildGraph.BuildDep}, BuildGraph.InstSet (bid,_)) -> begin
-            let srcpkg = CudfAdd.inttovar U.univ sid in
-            let binpkg = CudfAdd.inttovar U.univ bid in
+            let srcpkg = CudfAdd.inttopkg U.univ sid in
+            let binpkg = CudfAdd.inttopkg U.univ bid in
             if is_strong srcpkg binpkg then
               G.add_edge_e ag (v1,{BuildGraph.depend = BuildGraph.BuildDep; annotation = [`StrongDep]},v2)
             else
               G.add_edge_e ag (v1,label,v2)
           end
         | (BuildGraph.InstSet (bid,_), {BuildGraph.depend = BuildGraph.BuildsFrom {contents = s}}, _) -> begin
-            let binpkg = CudfAdd.inttovar U.univ bid in
+            let binpkg = CudfAdd.inttopkg U.univ bid in
             let binpkgs = IntSet.fold (fun id acc ->
-              (CudfAdd.inttovar U.univ id)::acc
+              (CudfAdd.inttopkg U.univ id)::acc
             ) s [] in
             if List.exists (fun pkg -> is_strong binpkg pkg) binpkgs then
               G.add_edge_e ag (v1,{BuildGraph.depend = BuildGraph.BuildsFrom {contents = s}; annotation = [`StrongDep]},v2)
