@@ -11,7 +11,7 @@
 (*  library, see the COPYING file for more information.                               *)
 (**************************************************************************************)
 
-open ExtLib
+open! ExtLib
 open Debian
 open Common
 open Algo
@@ -31,38 +31,38 @@ module Options = struct
 
   let dump = StdOpt.str_option ()
   let maforeign = StdOpt.store_true ()
-  let noindep = StdOpt.store_true ()
   let includextra = StdOpt.store_true ()
   let triplettable = StdOpt.str_option ()
   let cputable = StdOpt.str_option ()
-  let profiles = StdOptions.str_list_option ()
   let dropalternatives = StdOpt.store_true ()
 
   include StdOptions.DistcheckOptions ;;
   StdOptions.DistcheckOptions.add_options options ;;
 
   include StdOptions.InputOptions ;;
-  StdOptions.InputOptions.add_options options ;;
+  let default = 
+    List.fold_left (fun acc e ->
+      List.remove acc e
+    ) StdOptions.InputOptions.default_options ["inputtype";"fg";"bg";"compare"]
+  in
+  StdOptions.InputOptions.add_options ~default options ;
 
   include StdOptions.OutputOptions ;;
   StdOptions.OutputOptions.add_options options ;;
   StdOptions.OutputOptions.add_option options ~long_name:"dump" ~help:"dump the cudf file" dump;
 
   include StdOptions.DistribOptions ;;
-  StdOptions.DistribOptions.add_options options ;;
-  StdOptions.DistribOptions.add_option options ~long_name:"deb-triplettable"
+  StdOptions.DistribOptions.add_debian_options options ;;
+  let group = StdOptions.DistribOptions.deb_group options in
+  StdOptions.DistribOptions.add_option options ~group ~long_name:"deb-triplettable"
     ~help:"Path to an architecture triplet table like /usr/share/dpkg/triplettable" triplettable;
-  StdOptions.DistribOptions.add_option options ~long_name:"deb-cputable"
+  StdOptions.DistribOptions.add_option options ~group ~long_name:"deb-cputable"
     ~help:"Path to a cpu table like /usr/share/dpkg/cputable" cputable;
-  StdOptions.DistribOptions.add_option options ~long_name:"deb-defaulted-m-a-foreign"
+  StdOptions.DistribOptions.add_option options ~group ~long_name:"deb-defaulted-m-a-foreign"
     ~help:"Convert Arch:all packages to Multi-Arch: foreign" maforeign;
-  StdOptions.DistribOptions.add_option options ~long_name:"deb-drop-b-d-indep"
-    ~help:"Drop Build-Indep dependencies" noindep;
-  StdOptions.DistribOptions.add_option options ~long_name:"deb-include-extra-source"
+  StdOptions.DistribOptions.add_option options ~group ~long_name:"deb-include-extra-source"
     ~help:"Include packages with Extra-Source-Only:yes (dropped by default)" includextra;
-  StdOptions.DistribOptions.add_option options ~short_name:'P' ~long_name:"deb-profiles"
-    ~help:"comma separated list of activated build profiles" profiles;
-  StdOptions.DistribOptions.add_option options ~long_name:"deb-emulate-sbuild"
+  StdOptions.DistribOptions.add_option options ~group ~long_name:"deb-emulate-sbuild"
     ~help:"replicate sbuild behaviour to only keep the first alternative of build dependencies" dropalternatives;
 
 end
@@ -77,7 +77,7 @@ let progress = Util.Progress.create "Solver"
 let main () =
   let posargs = OptParse.OptParser.parse_argv Options.options in
   StdDebug.enable_debug (OptParse.Opt.get Options.verbose);
-  StdDebug.enable_timers (OptParse.Opt.get Options.timers) ["Solver"];
+  StdDebug.enable_timers (OptParse.Opt.get Options.timers) ["Solver";"Load.DebianSource"];
   StdDebug.enable_bars (OptParse.Opt.get Options.progress) ["Solver"];
   Util.Debug.disable "Depsolver_int";
   StdDebug.all_quiet (OptParse.Opt.get Options.quiet);
@@ -96,23 +96,17 @@ let main () =
   let options = Options.set_deb_options () in
   (* buildarch and native arch must be set to some architecture at this point *)
   let buildarch = Option.get options.Debian.Debcudf.native in
-  (* hostarch and noindep can be None *)
+  (* hostarch can be None *)
   let hostarch = match options.Debian.Debcudf.host with None -> "" | Some s -> s in
-  let noindep = OptParse.Opt.get Options.noindep in
+  let noindep = options.Debian.Debcudf.drop_bd_indep in
   let dropalternatives = OptParse.Opt.get Options.dropalternatives in
-
-  let profiles =
-    if OptParse.Opt.is_set Options.profiles then
-      OptParse.Opt.get Options.profiles
-    else
-      try String.nsplit (Sys.getenv "DEB_BUILD_PROFILES") " "
-      with Not_found -> []
-  in
+  let profiles = options.Debian.Debcudf.profiles in
 
   let filter_external_sources par =
     if (OptParse.Opt.get Options.includextra) then true
     else
-      try not(Debian.Packages.parse_bool (Debian.Packages.assoc "extra-source-only" par))
+      try not(Pef.Packages.parse_bool
+        ("extra-source-only",(Pef.Packages.assoc "extra-source-only" par)))
       with Not_found -> true
   in
 
@@ -135,31 +129,38 @@ let main () =
     |l -> 
         begin match List.rev l with
         |h::t ->
-          let srclist = StdLoaders.deb_load_source ~dropalternatives ~profiles ~filter:filter_external_sources ~noindep buildarch hostarch h in
+          let srclist =
+            StdLoaders.deb_load_source 
+              ~dropalternatives 
+              ~profiles 
+              ~filter:filter_external_sources 
+              ~noindep buildarch hostarch h 
+          in
           let pkglist = Deb.input_raw t in
           (pkglist,srclist)
         |_ -> fatal "An impossible situation occurred ?!#"
         end
   in
   let tables = Debcudf.init_tables (srclist @ pkglist) in
+  let global_constraints = Debian.Debcudf.get_essential ~options tables in
   let to_cudf (p,v) = (p,Debian.Debcudf.get_cudf_version tables (p,v)) in
-  let from_cudf (p,v) = (p,Debian.Debcudf.get_real_version tables (p,v)) in
-  let pp = CudfAdd.pp from_cudf in 
+  let from_cudf (p,v) = Debian.Debcudf.get_real_version tables (p,v) in
+  let pp = CudfAdd.pp ?fields:None ?decode:None from_cudf in 
 
   (* XXX here latest could be a bit faster if done at the same time of the cudf
      conversion *)
   let sl = 
     let l = List.map (fun pkg -> Debcudf.tocudf ~options tables pkg) srclist in
-    if OptParse.Opt.get Options.latest then
-      CudfAdd.latest l
+    if OptParse.Opt.is_set Options.latest then
+      CudfAdd.latest ~n:(OptParse.Opt.get Options.latest) l
     else
       l
   in
   let bl = 
     List.fold_left (fun acc pkg ->
       let pkg = 
-        if OptParse.Opt.get Options.maforeign && pkg.Packages.architecture = "all" then
-          { pkg with Packages.multiarch = `Foreign }
+        if OptParse.Opt.get Options.maforeign && pkg#architecture = "all" then
+          pkg#set_multiarch `Foreign
         else pkg
       in
       (Debcudf.tocudf ~options tables pkg)::acc
@@ -171,7 +172,11 @@ let main () =
 
   let failure = OptParse.Opt.get Options.failure in
   let success = OptParse.Opt.get Options.success in
-  let explain = OptParse.Opt.get Options.explain in
+  let explain =
+    if success || failure then
+      OptParse.Opt.get Options.explain
+    else false
+  in
   let minimal = OptParse.Opt.get Options.minimal in
   let summary = OptParse.Opt.get Options.summary in
 
@@ -179,7 +184,7 @@ let main () =
     if OptParse.Opt.is_set Options.checkonly then begin
       List.flatten (
         List.map (fun ((n,a),c) ->
-          let (name,filter) = Debian.Debutil.debvpkg to_cudf (("src:"^n,a),c) in
+          let (name,filter) = Pef.Pefcudf.pefvpkg to_cudf (("src:"^n,a),c) in
           Cudf.lookup_packages ~filter !universe name
         ) (OptParse.Opt.get Options.checkonly)
       )
@@ -189,18 +194,20 @@ let main () =
   Diagnostic.pp_out_version fmt;
 
   if OptParse.Opt.is_set Options.deb_native_arch then
-    Format.fprintf fmt "native-architecture: %s@." (OptParse.Opt.get Options.deb_native_arch);
+    Format.fprintf fmt "native-architecture: %s@."
+      (OptParse.Opt.get Options.deb_native_arch);
 
   if OptParse.Opt.is_set Options.deb_foreign_archs then
-    Format.fprintf fmt "foreign-architecture: %s@." (String.concat "," (OptParse.Opt.get Options.deb_foreign_archs));
+    Format.fprintf fmt "foreign-architecture: %s@."
+      (String.concat "," (OptParse.Opt.get Options.deb_foreign_archs));
 
   if OptParse.Opt.is_set Options.deb_host_arch then
-    Format.fprintf fmt "host-architecture: %s@." (OptParse.Opt.get Options.deb_host_arch);
+    Format.fprintf fmt "host-architecture: %s@."
+      (OptParse.Opt.get Options.deb_host_arch);
 
   let results = Diagnostic.default_result universe_size in
 
   if failure || success then Format.fprintf fmt "@[<v 1>report:@,";
-
   let failures = ref [] in
   let callback = function
     | {Diagnostic.result = Diagnostic.Failure _ ; request = [r]} -> failures := r::!failures
@@ -208,10 +215,11 @@ let main () =
       Diagnostic.fprintf ~pp ~failure ~success ~explain ~minimal fmt d
     | _ -> failwith "impossible"
   in
-  let nbp = Depsolver.listcheck ~callback !universe checklist in
+
+  Util.Timer.start timer;
+  let nbp = Depsolver.listcheck ~global_constraints ~callback ~explain !universe checklist in
 
   Util.Progress.set_total progress (List.length !failures);
-  Util.Timer.start timer;
   (* process the packages to check one by one because the changes to the
    * universe made for one package must not propagate to the others *)
   List.iter (fun pkg ->
@@ -220,7 +228,7 @@ let main () =
       (* reset the universe to its original state before every run *)
       universe := Cudf.load_universe bl;
       let rec aux () =
-        match Depsolver.listcheck ~callback:(BootstrapCommon.more_problems_callback (Diagnostic.fprintf ~pp ~failure ~success ~explain ~minimal fmt) universe results summary) !universe [pkg] with
+        match Depsolver.listcheck ~global_constraints ~callback:(BootstrapCommon.more_problems_callback (Diagnostic.fprintf ~pp ~failure ~success ~explain ~minimal fmt) universe results summary) !universe [pkg] with
         | 0 -> ()
         | _ -> aux ()
       in
@@ -235,8 +243,8 @@ let main () =
 
   let nb = universe_size in
   let nf = List.length sl in
-  Format.fprintf fmt "background-packages: %d@." nb;
-  Format.fprintf fmt "foreground-packages: %d@." (if nf = 0 then nb else nf);
+  Format.fprintf fmt "binary-packages: %d@." nb;
+  Format.fprintf fmt "source-packages: %d@." (if nf = 0 then nb else nf);
   Format.fprintf fmt "broken-packages: %d@." nbp;
 
   if summary then
