@@ -16,18 +16,18 @@
 (*                                                                            *)
 (******************************************************************************)
 
-open ExtLib
+open! ExtLib
 open Common
 open Algo
 open DoseparseNoRpm
 
 module Options = struct
   open OptParse
-  open OptParser
   let description = "Compute the list broken packages in a repository"
   let options = OptParser.make ~description
   include StdOptions.MakeOptions(struct let options = options end)
 
+  let lowmem = StdOpt.store_false ();;
   let coinst = StdDebian.vpkglist_option ();;
   let fields = StdOptions.str_list_option ();;
 
@@ -35,6 +35,7 @@ module Options = struct
   StdOptions.DistcheckOptions.add_options options ;;
   StdOptions.DistcheckOptions.add_option options ~long_name:"coinst" ~help:"Check if these packages are coinstallable" coinst;;
   StdOptions.DistcheckOptions.add_option options ~long_name:"fields" ~help:"Print additional fields if available" fields;;
+  StdOptions.DistcheckOptions.add_option options ~long_name:"lowmem" ~help:"Serialise multiple distcheck runs to save memory" lowmem;;
 
   include StdOptions.InputOptions ;;
   let default = "dot"::(StdOptions.InputOptions.default_options) in
@@ -44,8 +45,14 @@ module Options = struct
   StdOptions.OutputOptions.add_options options ;;
 
   include StdOptions.DistribOptions ;;
-  let default = List.remove StdOptions.DistribOptions.default_options "deb-host-arch" in
-  StdOptions.DistribOptions.add_options ~default options ;;
+  let default =
+    List.fold_left
+      List.remove
+      StdOptions.DistribOptions.default_options
+      ["deb-host-arch";"deb-drop-b-d-indep";"deb-profiles"]
+  in
+  StdOptions.DistribOptions.add_debian_options ~default options ;
+  StdOptions.DistribOptions.add_opam_options ~default options ;;
 
 end
 
@@ -74,25 +81,36 @@ let main () =
   let (input_type,implicit) = guess_format Options.inputtype inputlist in
 
   StdDebug.enable_debug (OptParse.Opt.get Options.verbose);
-  StdDebug.enable_timers (OptParse.Opt.get Options.timers) ["Solver"];
+  StdDebug.enable_timers (OptParse.Opt.get Options.timers) ["Solver";"Load"];
   StdDebug.enable_bars (OptParse.Opt.get Options.progress) ["Solver"];
   StdDebug.all_quiet (OptParse.Opt.get Options.quiet);
 
   let options = Options.set_options input_type in
   let (fg,bg) = Options.parse_cmdline (input_type,implicit) posargs in
 
-  let (preamble,pkgll,_,from_cudf,to_cudf,_) = StdLoaders.load_list ~options [fg;bg] in
-  let (fg_pkglist, bg_pkglist) = match pkgll with [fg;bg] -> (fg,bg) | _ -> assert false in
-  let fg_pkglist = 
-    if OptParse.Opt.get Options.latest then CudfAdd.latest fg_pkglist
+  let (_,pkgll,_,from_cudf,to_cudf,_,global_constraints) =
+    StdLoaders.load_list ~options [fg;bg]
+  in
+  let (fg_pkglist, bg_pkglist) =
+    match pkgll with [fg;bg] -> (fg,bg) | _ -> assert false
+  in
+  let fg_pkglist =
+    if OptParse.Opt.is_set Options.latest then
+      CudfAdd.latest ~n:(OptParse.Opt.get Options.latest) fg_pkglist
     else fg_pkglist
   in
+  let fn = List.length fg_pkglist in
+  let bn = List.length bg_pkglist in
   let bl =
     let s = CudfAdd.to_set (fg_pkglist @ bg_pkglist) in
     CudfAdd.Cudf_set.elements s
   in
   let universe = ref (Cudf.load_universe bl) in
   let universe_size = Cudf.universe_size !universe in
+  info "Cudf Universe: %d packages" universe_size;
+
+  (* we get back a bit of memory *)
+  if universe_size > 500000 then Gc.full_major () ;
 
   if OptParse.Opt.is_set Options.checkonly && 
     OptParse.Opt.is_set Options.coinst then
@@ -106,7 +124,7 @@ let main () =
         List.flatten (
           List.filter_map (fun ((n,a),c) ->
             try
-              let (name,filter) = Debian.Debutil.debvpkg to_cudf ((n,a),c) in
+              let (name,filter) = Pef.Pefcudf.pefvpkg to_cudf ((n,a),c) in
               Some(Cudf.lookup_packages ~filter !universe name)
             with Not_found -> None
           ) co
@@ -114,27 +132,25 @@ let main () =
       with 
       |[] ->
         fatal "Cannot find any package corresponding to the selector %s" 
-        (Debian.Printer.string_of_vpkglist co)
+        (Util.string_of_list ~sep:", " Pef.Printer.string_of_vpkg co)
       |l -> l
-    end else []
+    end else fg_pkglist
   in
 
   let coinstlist = 
     if OptParse.Opt.is_set Options.coinst then begin
       info "--coinst specified, consider all packages as background packages";
       let co = OptParse.Opt.get Options.coinst in
-      match
-        List.filter_map (fun ((n,a),c) ->
-          try
-            let (name,filter) = Debian.Debutil.debvpkg to_cudf ((n,a),c) in
-            Some(Cudf.lookup_packages ~filter !universe name)
-          with Not_found -> None
+      List.map (fun ((n,a),c) ->
+          match
+            let (name,filter) = Pef.Pefcudf.pefvpkg to_cudf ((n,a),c) in
+            Cudf.lookup_packages ~filter !universe name
+          with
+          |[] ->
+            fatal "Cannot find any package corresponding to the selector %s"
+              (Pef.Printer.string_of_vpkg ((n,a),c))
+          |l -> l
         ) co
-      with 
-      |[] ->
-        fatal "Cannot find any package corresponding to the selector %s" 
-        (Debian.Printer.string_of_vpkglist co)
-      |l -> l
     end else []
   in
   let fields = 
@@ -147,9 +163,16 @@ let main () =
   info "Solving..." ;
   let failure = OptParse.Opt.get Options.failure in
   let success = OptParse.Opt.get Options.success in
-  let explain = OptParse.Opt.get Options.explain in
+  let explain_summary = OptParse.Opt.get Options.explain in
   let minimal = OptParse.Opt.get Options.minimal in
+  let condense = OptParse.Opt.get Options.condense in
   let summary = OptParse.Opt.get Options.summary in
+  let explain =
+    if summary then true else
+    if success || failure then
+      OptParse.Opt.get Options.explain 
+    else false
+  in
   let fmt =
     if OptParse.Opt.is_set Options.outfile then
       let f =
@@ -169,25 +192,32 @@ let main () =
 
   Diagnostic.pp_out_version fmt;
 
+  if OptParse.Opt.is_set Options.deb_native_arch then
+    Format.fprintf fmt "native-architecture: %s@."
+	(OptParse.Opt.get Options.deb_native_arch);
+ 
+  if OptParse.Opt.is_set Options.deb_foreign_archs then
+    Format.fprintf fmt "foreign-architecture: %s@."
+	(String.concat "," (OptParse.Opt.get Options.deb_foreign_archs));
+ 
   if failure || success then Format.fprintf fmt "@[<v 1>report:@,";
+
   let callback d =
-    if summary then Diagnostic.collect results d ;
     let pp =
       if input_type = `Cudf then 
         fun pkg -> pp ~decode:(fun x -> x) pkg 
-      else fun pkg -> pp pkg
+      else fun pkg -> pp ?decode:None pkg
     in
-    let _ = 
-#ifdef HASOCAMLGRAPH 
-      if not(Diagnostic.is_solution d) && (OptParse.Opt.get Options.dot) then
-        let dir = OptParse.Opt.opt Options.outdir in
-        Diagnostic.print_dot ~pp ~addmissing:explain ?dir d
-#else
-      ()
-#endif
-    in
-    Diagnostic.fprintf ~pp ~failure ~success ~explain ~minimal fmt d
+    if summary then Diagnostic.collect results d ;
+
+    (if not(Diagnostic.is_solution d) && (OptParse.Opt.get Options.dot) then
+      let dir = OptParse.Opt.opt Options.outdir in
+      Diagnostic.print_dot ~pp ~addmissing:explain ?dir d);
+
+    if failure || success then
+      Diagnostic.fprintf ~pp ~failure ~success ~explain ~minimal ~condense fmt d
   in
+  let pp = pp ?decode:None in
   Util.Timer.start timer;
 
   if (OptParse.Opt.is_set Options.coinst) && (List.length coinstlist) > 0 then begin
@@ -201,7 +231,12 @@ let main () =
     Format.fprintf fmt "total-tuples: %d@." number_checks;
     Format.fprintf fmt "broken-tuples: %d@." nbt;
     nbt
-  end else begin 
+  end else begin
+    let univcheck = 
+      if OptParse.Opt.get Options.lowmem 
+      then Depsolver.univcheck
+      else Depsolver.univcheck_lowmem
+    in
     let failures = ref [] in
     let callback = function
       | {Diagnostic.result = Diagnostic.Failure _ ; request = [r]} -> failures := r::!failures
@@ -209,15 +244,12 @@ let main () =
         Diagnostic.fprintf ~pp ~failure ~success ~explain ~minimal fmt d
       | _ -> failwith "impossible"
     in
-    let global_constraints = not(OptParse.Opt.get Options.deb_ignore_essential) in
     let nbp =
       if (OptParse.Opt.is_set Options.checkonly) && (List.length checklist) = 0 then 0
-      else if OptParse.Opt.is_set Options.checkonly then 
-        Depsolver.listcheck ~global_constraints ~callback !universe checklist
-      else if bg_pkglist = [] then
-          Depsolver.univcheck ~global_constraints ~callback !universe 
+      else if OptParse.Opt.is_set Options.checkonly || not(bg_pkglist = []) then 
+        Depsolver.listcheck ~global_constraints ~callback ~explain !universe checklist
       else
-        Depsolver.listcheck ~global_constraints ~callback !universe fg_pkglist
+        univcheck ~global_constraints ~callback ~explain !universe 
     in
 
     Util.Progress.set_total progress (List.length !failures);
@@ -238,9 +270,6 @@ let main () =
     
     if failure || success then Format.fprintf fmt "@]@.";
     
-    let fn = List.length fg_pkglist in
-    let bn = List.length bg_pkglist in
-    
     let nb,nf = 
       let cl = List.length checklist in
       if cl != 0 then ((fn + bn) - cl,cl) else (bn,fn)
@@ -254,7 +283,7 @@ let main () =
     Format.fprintf fmt "total-packages: %d@." universe_size;
     Format.fprintf fmt "broken-packages: %d@." nbp;
     if summary then 
-      Format.fprintf fmt "@[%a@]@." (Diagnostic.pp_summary ~explain ~pp ()) results;
+      Format.fprintf fmt "@[%a@]@." (Diagnostic.pp_summary ~explain:explain_summary ~pp ()) results;
     nbp
   end
 ;;
