@@ -11,7 +11,7 @@
 (*  library, see the COPYING file for more information.                   *)
 (**************************************************************************)
 
-open ExtLib
+open! ExtLib
 open Common
 
 module IntSet = BootstrapCommon.IntSet
@@ -20,20 +20,20 @@ module Make (U : sig val univ : Cudf.universe end) = struct
   module G = SrcGraph.G
 
   let string_of_intsetscc s =
-    let minpkg = CudfAdd.inttovar U.univ (IntSet.min_elt s) in
+    let minpkg = CudfAdd.inttopkg U.univ (IntSet.min_elt s) in
     let pkgname = BootstrapCommon.string_of_package minpkg in
     let card = IntSet.cardinal s in
     Printf.sprintf "%s, ... and %d more" pkgname (card-1)
 
   let package_of_vertex = function
     | SrcGraph.SrcPkg id ->
-        CudfAdd.inttovar U.univ id
+        CudfAdd.inttopkg U.univ id
     | SrcGraph.SCC s ->
-        CudfAdd.inttovar U.univ (IntSet.min_elt s)
+        CudfAdd.inttopkg U.univ (IntSet.min_elt s)
 
   let string_of_vertex ?(fvs=IntSet.empty) = function
     | SrcGraph.SrcPkg id ->
-        let pkg = CudfAdd.inttovar U.univ id in
+        let pkg = CudfAdd.inttopkg U.univ id in
         let name = BootstrapCommon.string_of_package pkg in
         if IntSet.mem id fvs then name^"(*)" else name
     | SrcGraph.SCC s ->
@@ -44,9 +44,9 @@ module Make (U : sig val univ : Cudf.universe end) = struct
     G.iter_edges_e (fun (v1,label,v2) ->
       match v1 with
         | SrcGraph.SrcPkg sid ->
-            let srcpkg = CudfAdd.inttovar U.univ sid in
+            let srcpkg = CudfAdd.inttopkg U.univ sid in
             let binaries = !(label.SrcGraph.binaries) in
-            let strong = IntSet.filter (fun pid -> is_strong srcpkg (CudfAdd.inttovar U.univ pid)) binaries in
+            let strong = IntSet.filter (fun pid -> is_strong srcpkg (CudfAdd.inttopkg U.univ pid)) binaries in
             if IntSet.is_empty strong then
               G.add_edge_e ag (v1,label,v2)
             else begin
@@ -63,11 +63,11 @@ module Make (U : sig val univ : Cudf.universe end) = struct
                  * dependencies found above *)
                 let inters = List.fold_left (fun acc pkg ->
                   let strong2 = IntSet.fold (fun pid acc ->
-                    if is_strong pkg (CudfAdd.inttovar U.univ pid) then IntSet.add pid acc else acc
+                    if is_strong pkg (CudfAdd.inttopkg U.univ pid) then IntSet.add pid acc else acc
                   ) strong IntSet.empty in
                   IntSet.inter strong2 acc
                 ) strong pkgs in
-                let pkgs = List.map (CudfAdd.vartoint U.univ) pkgs in
+                let pkgs = List.map (CudfAdd.pkgtoint U.univ) pkgs in
                 (* if the intersection of those sets is not empty, add the
                  * packages of the disjuction *)
                 if IntSet.is_empty inters then acc
