@@ -1,9 +1,19 @@
 #!/bin/sh -e
 
+# to make sure that lines like "rm $tmp/*" doesn't remove root
+set -u
+
 PROGNAME=`basename $0`
-CUSTOMTMP="yes"
+CUSTOMTMP="no"
 OUTDIR="./"
+KEEP="no"
 NUM_CORES="1"
+DEVELOP="no"
+TIMERS=
+ALLOWSRCMISMATCH=
+IGNORESRCLESSBIN=
+SAPSB=
+ONLINE=
 
 usage() {
 	echo "usage: $PROGNAME [OPTIONS] [buildarch hostarch buildpackages sources]" >&2
@@ -12,13 +22,18 @@ usage() {
 	echo " -k, --keep       Keep the temporary files" >&2
 	echo " -w, --online     Produce stat.html for online consumption" >&2
 	echo " -o, --output=DIR Output directory. Default is the current directory" >&2
-	echo " -t, --tmp=DIR    Temporary directory. Default is created by mktemp(1)" >&2
+	echo " -t, --tmp=DIR    Temporary directory. Default is created by mktemp(1). Implies --keep." >&2
 	echo " -G, --optgraph   Calculate a dependency graph where each installation" >&2
 	echo "                  set contains the minimal number of unavailable" >&2
 	echo "                  binary packages" >&2
+	echo " -T, --timers           time all program executions" >&2
+	echo " -m, --allowsrcmismatch Allow binary packages to be associated with a source " >&2
+	echo "                        package of a different version than themselves" >&2
+	echo " -e, --ignoresrclessbin Ignore binary packages without a source package" >&2
+	echo " -b, --sapsb            Calculate strong articulation points and bridges" >&2
 	echo " -j, --jobs=NUM   Number of threads or processes to run in parallel" >&2
 	echo "                  where applicable" >&2
-	echo " -I, --drop-b-d-indep Drop Build-Depends-Indep dependencies" >&2
+	echo " -I, --deb-drop-b-d-indep Drop Build-Depends-Indep dependencies" >&2
 	echo " -v, --verbose    Be more verbose" >&2
 	echo " -d, --debug      Maximum verbosity" >&2
 	echo " -D, --develop    Execute tools from the source checkout instead of \$PATH" >&2
@@ -26,7 +41,7 @@ usage() {
 
 die() {
         echo " + error $@"
-        if [ -n "$KEEP" ] || [ "$CUSTOMTMP" = "yes" ]; then
+        if [ "$KEEP" = "yes" ]; then
                 echo " + temporary files stored in $tmp" >&2
         else
                 rm -f "$tmp"/*
@@ -49,10 +64,10 @@ trap "die \"got signal\"" 1 2 15
 getopt -T > /dev/null && exitcode=0 || exitcode=$?
 if [ $exitcode -eq 4 ]; then
 	# GNU enhanced getopt is available
-	ARGS=`getopt --long help,online,output:,tmp:,optgraph,jobs:,drop-b-d-indep,verbose,debug,develop --options hwo:t:Gj:IvdD -- "$@"`
+	ARGS=`getopt --long help,output:,tmp:,optgraph,timers,allowsrcmismatch,ignoresrclessbin,sapsb,online,jobs:,deb-drop-b-d-indep,verbose,debug,develop --options ho:t:GTmebwj:IvdD -- "$@"`
 else
 	# Original getopt is available
-	ARGS=`getopt hwo:t:Gj:IvdD "$@"`
+	ARGS=`getopt ho:t:GTmebwj:IvdD "$@"`
 fi
 
 if [ $? -ne 0 ]; then
@@ -64,45 +79,60 @@ eval set -- $ARGS
 while [ $# -gt 0 ]; do
 	case "$1" in
 		-h | --help)    usage; exit 0;;
-		-k | --keep)    KEEP=true;;
+		-k | --keep)    KEEP="yes";;
 		-w | --online)  ONLINE="--online";;
 		-o | --output)  OUTDIR="$2"; shift;;
-		-t | --tmp)     tmp="$2"; shift;;
+		-t | --tmp)     CUSTOMTMP="yes"; KEEP="yes"; tmp="$2"; shift;;
 		-G | --optgraph) OPTGRAPH=--optgraph;;
+		-T | --timers)   TIMERS=--timers;;
+		-m | --allowsrcmismatch) ALLOWSRCMISMATCH=--allowsrcmismatch;;
+		-e | --ignoresrclessbin) IGNORESRCLESSBIN=--ignoresrclessbin;;
+		-b | --sapsb)    SAPSB=--sapsb;;
 		-j | --jobs)    NUM_CORES="$2"; shift;;
-		-I | --drop-b-d-indep) DROPBDINDEP=--drop-b-d-indep;;
+		-I | --deb-drop-b-d-indep) DROPBDINDEP=--deb-drop-b-d-indep;;
 		-v | --verbose) VERBOSE=--verbose;;
 		-d | --debug)   set -x;;
-		-D | --develop) DEVELOP=true;;
+		-D | --develop) DEVELOP="yes";;
 		--)             shift; break;;
 	esac
 	shift
 done
 
+getocamlexec() {
+	execname="$1"
+	for ext in "native" "p.native" "byte" "d.byte"; do
+		path="./$execname.$ext"
+		if [ -f "$path" ]; then
+			echo "$path"
+			return
+		fi
+	done
+	die "cannot find executable for $execname"
+}
+
 bin_coinstall=dose-deb-coinstall
 bin_buildcheck=dose-builddebcheck
-if [ -n "$DEVELOP" ]; then
+if [ "$DEVELOP" = "yes" ]; then
 	droppable=./droppable
-	[ -f ./bin2src.native ] && bin_bin2src=./bin2src.native || bin_bin2src=./bin2src.d.byte
-	[ -f ./build-fixpoint.native ] && bin_build_fixpoint=./build-fixpoint.native || bin_build_fixpoint=./build-fixpoint.d.byte
-	[ -f ./buildgraph2srcgraph.native ] && bin_buildgraph_to_srcgraph=./buildgraph2srcgraph.native || bin_buildgraph_to_srcgraph=./buildgraph2srcgraph.d.byte
-	[ -f ./clean-repository.native ] && bin_clean_repository=./clean-repository.native || bin_clean_repository=./clean-repository.d.byte
-	[ -f ./create-graph.native ] && bin_create_graph=./create-graph.native || bin_create_graph=./create-graph.d.byte
-	[ -f ./find-fvs.native ] && bin_find_fvs=./find-fvs.native || bin_find_fvs=./find-fvs.d.byte
-	[ -f ./partial-order.native ] && bin_partial_order=./partial-order.native || bin_partial_order=./partial-order.d.byte
-	[ -f ./print-stats.native ] && bin_print_stats=./print-stats.native || bin_print_stats=./print-stats.d.byte
-	[ -f ./src2bin.native ] && bin_src2bin=./src2bin.native || bin_src2bin=./src2bin.d.byte
+	bin_bin2src=$(getocamlexec bin2src)
+	bin_build_fixpoint=$(getocamlexec build-fixpoint)
+	bin_buildgraph_to_srcgraph=$(getocamlexec buildgraph2srcgraph)
+	bin_clean_repository=$(getocamlexec clean-repository)
+	bin_create_graph=$(getocamlexec create-graph)
+	bin_find_fvs=$(getocamlexec find-fvs)
+	bin_partial_order=$(getocamlexec partial-order)
+	bin_print_stats=$(getocamlexec print-stats)
+	bin_src2bin=$(getocamlexec src2bin)
+	bin_buildcheck_more_problems=$(getocamlexec buildcheck-more-problems)
 	bin_add_arch=./tools/add-arch.py
 	bin_apply_ma_diff=./tools/apply-ma-diff.py
 	bin_convert_arch=./tools/convert-arch.py
-	bin_dose2html=./tools/dose2html.py
 	bin_extract_scc=./tools/extract-scc.py
 	bin_fasofstats=./tools/fasofstats.py
 	bin_fix_cross_problems=./tools/fix-cross-problems.py
 	bin_graph_info=./tools/graph-info.py
 	bin_ma_diff=./tools/ma-diff.py
 	bin_packages_difference=./tools/packages-difference.py
-	bin_packages_intersection=./tools/packages-intersection.py
 	bin_packages_union=./tools/packages-union.py
 	bin_profile_build_fvs=./tools/profile-build-fvs.py
 	bin_stat_html=./tools/stat-html.py
@@ -119,17 +149,16 @@ else
 	bin_partial_order=botch-partial-order
 	bin_print_stats=botch-print-stats
 	bin_src2bin=botch-src2bin
+	bin_buildcheck_more_problems=botch-buildcheck-more-problems
 	bin_add_arch=botch-add-arch
 	bin_apply_ma_diff=botch-apply-ma-diff
 	bin_convert_arch=botch-convert-arch
-	bin_dose2html=botch-dose2html
 	bin_extract_scc=botch-extract-scc
 	bin_fasofstats=botch-fasofstats
 	bin_fix_cross_problems=botch-fix-cross-problems
 	bin_graph_info=botch-graph-info
 	bin_ma_diff=botch-ma-diff
 	bin_packages_difference=botch-packages-difference
-	bin_packages_intersection=botch-packages-intersection
 	bin_packages_union=botch-packages-union
 	bin_profile_build_fvs=botch-profile-build-fvs
 	bin_stat_html=botch-stat-html
@@ -137,9 +166,6 @@ else
 	bin_buildgraph2packages=botch-buildgraph2packages
 fi
 
-
-mkdir -p "$OUTDIR"
-
 case $# in
 	0)
 		BUILDARCH=amd64
@@ -163,14 +189,15 @@ case $# in
 		;;
 esac
 
-if [ -n "$tmp" ]; then
-	mkdir -p "$tmp"
+if [ $CUSTOMTMP = "yes" ]; then
+	mkdir -p "$tmp" || die "cannot mkdir $tmp"
 else
 	tmp=`mktemp --directory`
-	CUSTOMTMP="no"
 fi
 
-if [ "$KEEP" != "true" ] && [ -n "$(ls -A $tmp)" ]; then
+mkdir -p "$OUTDIR" || die "cannot mkdir $OUTDIR"
+
+if [ "$KEEP" != "yes" ] && [ -n "$(ls -A $tmp)" ]; then
        echo "$tmp is not empty and you did not specify --keep" >&2
        echo "refusing to run and delete that directory" >&2
        exit 1
@@ -180,13 +207,17 @@ echo " + running clean repository" >&2
 
 # clean up buildpackages repository
 $bin_clean_repository $VERBOSE $DROPBDINDEP --deb-native-arch=$BUILDARCH \
-	$buildpackages $sources \
+	--progress $buildpackages $sources \
 	> $tmp/buildpackages_clean
 buildpackages="$tmp/buildpackages_clean"
 
+echo " + running grep-dctrl" >&2
+
+run grep-dctrl --not -X -FArchitecture all "$buildpackages" > "$buildpackages.noarchall" || die "grep-dctrl failed"
+
 echo " + running bin2src" >&2
 
-run $bin_bin2src $VERBOSE --noarchall --deb-native-arch=$BUILDARCH $buildpackages $sources \
+run $bin_bin2src $VERBOSE --deb-native-arch=$BUILDARCH $buildpackages.noarchall $sources \
 	> $tmp/sources_clean || die "bin2src failed"
 sources="$tmp/sources_clean"
 
@@ -216,10 +247,14 @@ run $bin_coinstall $VERBOSE \
     --bg="$hostpackages" --fg="$tmp/minimal" --deb-native-arch=$HOSTARCH \
     > "$tmp/minimal-$HOSTARCH" || die "coinstall failed"
 
+echo " + running grep-dctrl" >&2
+
+run grep-dctrl --not -X -FArchitecture all "$tmp/minimal-$HOSTARCH" > "$tmp/minimal-$HOSTARCH.noarchall" || die "grep-dctrl failed"
+
 echo " + running bin2src..." >&2
 
-run $bin_bin2src $VERBOSE --noarchall --deb-native-arch=$HOSTARCH $ALLOWSRCMISMATCH \
-        "$tmp/minimal-$HOSTARCH" "$sources" \
+run $bin_bin2src $VERBOSE --deb-native-arch=$HOSTARCH $ALLOWSRCMISMATCH \
+        "$tmp/minimal-$HOSTARCH.noarchall" "$sources" \
         > "$tmp/crossed-src" || die "bin2src failed"
 
 echo " + running grep-dctrl..." >&2
@@ -234,7 +269,7 @@ echo " + creating a self-contained repository graph..." >&2
 # $tmp/minimal-$HOSTARCH
 # the graph is started from all source packages that have been crossed before
 # and are thus creating the binary packages in $tmp/minimal-$HOSTARCH
-run $bin_create_graph $VERBOSE $DROPBDINDEP --num_cores=$NUM_CORES \
+run $bin_create_graph $VERBOSE $DROPBDINDEP --jobs=$NUM_CORES \
 	--deb-ignore-essential --progress --timers \
 	-A "$tmp/available-all" $ALLOWSRCMISMATCH --deb-native-arch=$HOSTARCH \
 	--bg $sources $hostpackages $tmp/crossed-src \
@@ -251,10 +286,14 @@ run $bin_packages_union $VERBOSE "$tmp/minimal-closure" \
 	"$tmp/minimal-$HOSTARCH" "$tmp/minimal-closure" \
 	|| die "union failed"
 
+echo " + running grep-dctrl" >&2
+
+run grep-dctrl --not -X -FArchitecture all "$tmp/minimal-closure" > "$tmp/minimal-closure.noarchall" || die "grep-dctrl failed"
+
 echo " + running bin2src..." >&2
 
-run $bin_bin2src $VERBOSE --noarchall --deb-native-arch=$HOSTARCH $ALLOWSRCMISMATCH \
-       "$tmp/minimal-closure" "$sources" \
+run $bin_bin2src $VERBOSE --deb-native-arch=$HOSTARCH $ALLOWSRCMISMATCH \
+       "$tmp/minimal-closure.noarchall" "$sources" \
        > "$tmp/minimal-closure-src" || die "bin2src failed"
 
 hostpackages="$tmp/minimal-closure"
@@ -265,7 +304,7 @@ echo " + running buildcheck..." >&2
 # if buildcheck succeeds and since the source list was generated from the
 # binary list, at this point we know that the pair of minimal-closure and
 # minimal-closure-src is indeed a valid self-contained repository
-run $bin_buildcheck $VERBOSE --explain-minimal \
+run $bin_buildcheck $VERBOSE --explain-minimal --deb-drop-b-d-indep \
 	--failures --deb-native-arch=$HOSTARCH "$hostpackages" \
 	"$sources" || die "buildcheck failed"
 
@@ -296,27 +335,20 @@ fi
 # fix the remaining problems
 cp $buildpackages $tmp/buildpackages-fixed
 cp $hostpackages $tmp/minimal-closure-crossable-fixed
-i=0
-while true; do
-	echo $i
-	i=$((i+1))
-	run $bin_buildcheck $VERBOSE --explain --failures \
-		--deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH \
-		$tmp/minimal-closure-crossable-fixed $tmp/buildpackages-fixed \
-		"$tmp/crossbuild-essential-${HOSTARCH}" $sources \
-		> $tmp/buildcheck.yaml || [ $? -lt 64 ] || die "buildcheck failed"
-	run $bin_packages_union $VERBOSE $tmp/buildpackages-fixed $tmp/buildpackages-fixed || die "union failed"
-	run $bin_packages_union $VERBOSE $tmp/minimal-closure-crossable-fixed $tmp/minimal-closure-crossable-fixed || die "union failed"
-	oldmd5=`cat $tmp/buildpackages-fixed $tmp/minimal-closure-crossable-fixed | md5sum`
-	run $bin_fix_cross_problems $VERBOSE $tmp/buildcheck.yaml $tmp/buildpackages-fixed \
-		$tmp/minimal-closure-crossable-fixed || die "fix-cross-problems failed"
-	run $bin_ma_diff $VERBOSE $oldbuildpackages $tmp/buildpackages-fixed \
-		| sort > "${OUTDIR}/ma.diff" || true # non-zero exit is not fatal here
-	run $bin_packages_union $VERBOSE $tmp/buildpackages-fixed $tmp/buildpackages-fixed || die "union failed"
-	run $bin_packages_union $VERBOSE $tmp/minimal-closure-crossable-fixed $tmp/minimal-closure-crossable-fixed || die "union failed"
-	newmd5=`cat $tmp/buildpackages-fixed $tmp/minimal-closure-crossable-fixed | md5sum`
-	[ "$oldmd5" = "$newmd5" ] && break
-done
+
+run $bin_buildcheck_more_problems $VERBOSE --explain --failures \
+	--deb-drop-b-d-indep --deb-native-arch=$BUILDARCH \
+	--deb-host-arch=$HOSTARCH $tmp/minimal-closure-crossable-fixed \
+	$tmp/buildpackages-fixed "$tmp/crossbuild-essential-${HOSTARCH}" \
+	$sources \
+	> $tmp/buildcheck.yaml || [ $? -lt 64 ] || die "buildcheck failed"
+run $bin_fix_cross_problems $VERBOSE $tmp/buildcheck.yaml $tmp/buildpackages-fixed \
+	$tmp/minimal-closure-crossable-fixed || die "fix-cross-problems failed"
+run $bin_ma_diff $VERBOSE $oldbuildpackages $tmp/buildpackages-fixed \
+	| sort > "${OUTDIR}/ma.diff" || true # non-zero exit is not fatal here
+run $bin_packages_union $VERBOSE $tmp/buildpackages-fixed $tmp/buildpackages-fixed || die "union failed"
+run $bin_packages_union $VERBOSE $tmp/minimal-closure-crossable-fixed $tmp/minimal-closure-crossable-fixed || die "union failed"
+
 buildpackages=$tmp/buildpackages-fixed
 hostpackages=$tmp/minimal-closure-crossable-fixed
 
@@ -341,12 +373,12 @@ run $bin_packages_difference $VERBOSE "$hostpackages" "$tmp/available-all" \
 # we delete all m-a:foreign packages from the host package list because
 # arch:all packages come from the build package list
 echo " + running grep-dctrl..." >&2
-run grep-dctrl -X \( -FArchitecture all --or --not -FMulti-Arch foreign \) "$hostpackages" \
+run grep-dctrl -X \( --not -FArchitecture all --and --not -FMulti-Arch foreign \) "$hostpackages" \
 	> "$tmp/hostpackages-no-ma-foreign" || die "grep-dctrl failed"
 hostpackages="$tmp/hostpackages-no-ma-foreign"
 
 echo " + bin2src..." >&2
-run $bin_bin2src $VERBOSE --noarchall --deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH $ALLOWSRCMISMATCH \
+run $bin_bin2src $VERBOSE --deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH $ALLOWSRCMISMATCH \
         "$tmp/not-available" "$sources" \
         > "$tmp/tocompile" || die "bin2src failed"
 
@@ -367,9 +399,10 @@ run $bin_build_fixpoint $VERBOSE $DROPBDINDEP --deb-native-arch=$BUILDARCH --deb
 
 echo " + running src2bin..." >&2
 
+# no need to pass available packages here because there are no arch:all
+# packages in the list of hostarch Packages file
 run $bin_src2bin $VERBOSE --deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH $ALLOWSRCMISMATCH \
-    $IGNORESRCLESSBIN -A "$tmp/available-all" --bg "$sources" \
-    "$hostpackages" "$tmp/compilable-src" \
+    $IGNORESRCLESSBIN --bg "$sources" "$hostpackages" "$tmp/compilable-src" \
     > "$tmp/compilable" || die "src2bin failed"
 
 echo " + union..." >&2
@@ -390,7 +423,7 @@ UNIVERSE="--deb-native-arch=$BUILDARCH --deb-host-arch=$HOSTARCH --bg $sources $
 # was made sure to contain a coinstallation set of essential earlier in this
 # script
 run $bin_create_graph $VERBOSE $DROPBDINDEP $OPTGRAPH --progress --timers \
-	--num_cores=$NUM_CORES --deb-ignore-essential $STRONGARG \
+	--jobs=$NUM_CORES --deb-ignore-essential \
 	-A "$tmp/available" $ALLOWSRCMISMATCH $UNIVERSE \
 	> "${OUTDIR}/buildgraph.xml" || die "create graph failed"
 
@@ -403,7 +436,7 @@ droplist="--remove-weak=${droppable}/weak-build-dependencies.list --remove-reduc
 
 run $bin_print_stats $VERBOSE $DROPBDINDEP --max-length=4 --max-length-fas=8 $ALLOWSRCMISMATCH \
 	$SAPSB $droplist -A "$tmp/available" "${OUTDIR}/buildgraph.xml" \
-	"${OUTDIR}/srcgraph.xml" $UNIVERSE > "${OUTDIR}/stats.json" || die "print stats failed"
+	"${OUTDIR}/srcgraph.xml" $UNIVERSE | ydump > "${OUTDIR}/stats.json" || die "print stats failed"
 
 echo " + generating html..." >&2
 
@@ -415,8 +448,9 @@ run $bin_fasofstats $VERBOSE "${OUTDIR}/stats.json" | sort > "${OUTDIR}/remove.l
 
 echo " + extract strongly connected components..." >&2
 
-run $bin_extract_scc $VERBOSE --outdir=${OUTDIR} "${OUTDIR}/buildgraph.xml" \
-	cyclic --outfnameattr=cudfname --outfnameverts type:src \
+run $bin_extract_scc $VERBOSE --outdir=${OUTDIR} \
+	--outfnameattr=cudfname --outfnamevert type:src \
+	"${OUTDIR}/buildgraph.xml" cyclic \
 	| sort > "${OUTDIR}/fas" || die "extract scc failed"
 
 echo " + calculate feedback vertex sets..." >&2
@@ -457,7 +491,7 @@ while read g; do
 	run $bin_graph_info $VERBOSE "${OUTDIR}/$g" || die "graph-info failed"
 done < "${OUTDIR}/fas"
 
-if [ -n "$KEEP" ] || [ "$CUSTOMTMP" = "yes" ]; then
+if [ "$KEEP" = "yes" ]; then
 	echo " + temporary files stored in $tmp" >&2
 else
 	rm -f "$tmp"/*
