@@ -4,6 +4,7 @@ from __future__ import print_function
 from collections import defaultdict
 from functools import cmp_to_key
 import sys
+import argparse
 sys.path.append('/usr/share/botch')
 from util import write_plain, read_yaml_file, cmp, parse_dose_yaml_mc
 from util import read_tag_file
@@ -11,7 +12,7 @@ import os
 
 
 def print_package(arg, bin2src, source=None, color=True):
-    pkgname, version, vpkg = arg
+    pkgname, arch, version, vpkg = arg
     sn, _ = src_of_pkg(arg, bin2src)
     if source:
         highlight = (sn == source[0])
@@ -29,10 +30,7 @@ def print_package(arg, bin2src, source=None, color=True):
         else:
             pkgstring = 'src:%s' % pkgname
     else:
-        pkgname, arch = pkgname.split(':', 1)
         bugs = binpkgbugs.get(pkgname)
-        if bin2src.get((pkgname, 'all', version)):
-            arch = 'all'
         if version:
             pkgstring = '<a title="%s:%s (= %s)">%s</a>' % (pkgname, arch,
                                                             version, pkgname)
@@ -285,11 +283,14 @@ def print_conflict(outfile, conflict, bin2src, source=None):
     print("</table>", file=outfile)
 
 
-def print_footer(outfile, timestamp, srcs=[], srcsdir=""):
+def print_footer(outfile, timestamp, srcs=[], srcsdir="", wwwroot=""):
     if srcs:
         print("<h2>Affected source packages:</h2>", file=outfile)
+        srcsdir += "/"
+        if srcsdir.startswith(wwwroot):
+            srcsdir = srcsdir[len(wwwroot):]
         for src in sorted([s for s in srcs if s is not None]):
-            print("<a href=\"%s/%s.html\">%s</a>" % (srcsdir, src, src),
+            print("<a href=\"%s%s.html\">%s</a>" % (srcsdir, src, src),
                   file=outfile)
 
     url = \
@@ -317,16 +318,12 @@ def print_footer(outfile, timestamp, srcs=[], srcsdir=""):
 
 
 def src_of_pkg(t, bin2src):
-    n, v, _ = t
+    n, a, v, _ = t
     if n.startswith("src:"):
         return n[4:], v
     if n == "":
         return (None, None)
-    n, a = n.split(":", 1)
     res = bin2src.get((n, a, v))
-    if res:
-        return res
-    res = bin2src.get((n, 'all', v))
     if res:
         return res
     # this can happen for a dependency on a package that is not in the archive
@@ -425,11 +422,11 @@ def create_srcpkg_pages(missing, conflict, bin2src, srcsdir, description,
         if not srcpkg:
             continue
         fname = os.path.join(srcsdir, srcpkg) + ".html"
-        with open(fname, "w") as srcpkgpage:
+        with open(fname, "w", encoding="utf8") as srcpkgpage:
             print_header(srcpkgpage, description, btsuser, btstag)
             for ver in versions.keys():
                 print("<h1>%s</h1>" % print_package(
-                    ("src:" + srcpkg, ver, None), bin2src, color=False),
+                    ("src:" + srcpkg, None, ver, None), bin2src, color=False),
                     file=srcpkgpage)
                 print_top10summary(srcpkgpage, versions[ver].get('missing'),
                                    versions[ver].get('conflict'), bin2src,
@@ -447,7 +444,7 @@ def create_srcpkg_pages(missing, conflict, bin2src, srcsdir, description,
 
 def dose2html(yamlin, outfile, description="", srcpkgbugs=set(),
               binpkgbugs=set(), btsuser=None, btstag=None, srcsdir=None,
-              packages=[], timestamp=None, verbose=False):
+              packages=[], timestamp=None, wwwroot=None, verbose=False):
     bin2src = dict()
 
     for pkg in packages:
@@ -485,14 +482,33 @@ def dose2html(yamlin, outfile, description="", srcpkgbugs=set(),
     if srcsdir and packages:
         srcpkgs = create_srcpkg_pages(missing, conflict, bin2src, srcsdir,
                                       description, btsuser, btstag, timestamp)
-        print_footer(outfile, timestamp, srcpkgs, srcsdir)
+        print_footer(outfile, timestamp, srcpkgs, srcsdir, wwwroot)
     else:
         print_footer(outfile, timestamp)
 
+
+class _ExtendAction(argparse.Action):
+    def __init__(self, option_strings, dest, nargs=None, const=None, **kwargs):
+        if nargs == 0:
+            raise ValueError('nargs for extend actions must be > 0')
+        if const is not None and nargs != argparse.OPTIONAL:
+            raise ValueError('nargs must be %r to supply const'
+                             % argparse.OPTIONAL)
+        super(_ExtendAction, self).__init__(
+            option_strings=option_strings, dest=dest, nargs=nargs, const=const,
+            **kwargs)
+
+    def __call__(self, parser, namespace, values, option_string=None):
+        items = argparse._copy.copy(argparse._ensure_value(namespace,
+                                                           self.dest, []))
+        items.extend(values)
+        setattr(namespace, self.dest, items)
+
+
 if __name__ == '__main__':
-    import argparse
     parser = argparse.ArgumentParser(
         description='given a buildcheck result, create a html overview')
+    parser.register('action', 'extend', _ExtendAction)
     parser.add_argument('yamlin', type=read_yaml_file,
                         help='input in yaml format (dose3 output)')
     parser.add_argument(
@@ -506,15 +522,25 @@ if __name__ == '__main__':
                         'individual source package overviews')
     parser.add_argument('--wnpp', action='store_true',
                         help='retrieve and print wnpp status for packages')
-    parser.add_argument('--packages', type=read_tag_file, default=[],
+    parser.add_argument('--packages', type=read_tag_file, action='extend',
+                        default=[],
                         help='Packages file to create a mapping from binary ' +
                              'to source packages')
     parser.add_argument('--timestamp',
                         help='put a freeform timestamp string at the bottom ' +
                              'of the generated HTML')
+    parser.add_argument('--wwwroot',
+                        help='The HTML hyperlink to the source package '
+                        'overview by default will begin with the string given '
+                        'in the "--srcsdir" option. The string given by this '
+                        'option will be removed from the beginning of that '
+                        'path in the HTML href attribute. If this option ends '
+                        'with a slash, then the resulting hyperlinks will '
+                        'become relative.')
     parser.add_argument(
         '-v', '--verbose', action='store_true', help='be verbose')
     args = parser.parse_args()
+
     srcpkgbugs = defaultdict(list)
     binpkgbugs = defaultdict(list)
 
@@ -577,4 +603,4 @@ if __name__ == '__main__':
     with args.htmlout as f:
         dose2html(args.yamlin, f, args.desc, srcpkgbugs, binpkgbugs,
                   args.btsuser, args.btstag, args.srcsdir, args.packages,
-                  args.timestamp, args.verbose)
+                  args.timestamp, args.wwwroot, args.verbose)
