@@ -4,15 +4,15 @@ from __future__ import print_function
 import networkx as nx
 import sys
 sys.path.append('/usr/share/botch')
-from util import read_graphml
+from util import read_graph
 
 if __name__ == "__main__":
     import argparse
     parser = argparse.ArgumentParser(
         description=("Print type and number of vertices, edges and SCC of a " +
-                     "graph in GraphML format"))
+                     "graph in GraphML or dot format"))
     parser.add_argument(
-        "g", type=read_graphml, help="Input graph in GraphML format")
+        "g", type=read_graph, help="Input graph in GraphML or dot format")
     parser.add_argument(
         '-v', '--verbose', action='store_true', help='be verbose')
     args = parser.parse_args()
