@@ -12,6 +12,8 @@ try:
     from yaml import CBaseLoader as yamlLoader
 except ImportError:
     from yaml import BaseLoader as yamlLoader
+import argparse
+import xml
 
 
 def write_plain(path):
@@ -35,11 +37,27 @@ def write_plain(path):
 
 
 def read_yaml_file(path):
-    if path == '-':
-        data = yaml.load(sys.stdin, Loader=yamlLoader)
-    else:
-        with open(path) as f:
-            data = yaml.load(f, Loader=yamlLoader)
+    try:
+        if path == '-':
+            data = yaml.load(sys.stdin, Loader=yamlLoader)
+        else:
+            try:
+                with open(path) as f:
+                    data = yaml.load(f, Loader=yamlLoader)
+            except IsADirectoryError:
+                raise argparse.ArgumentTypeError(
+                    "\"%s\" is a directory" % path)
+            except PermissionError:
+                raise argparse.ArgumentTypeError(
+                    "\"%s\" permission denied" % path)
+            except FileNotFoundError:
+                raise argparse.ArgumentTypeError(
+                    "\"%s\" does not exist" % path)
+    except yaml.scanner.ScannerError as e:
+        raise argparse.ArgumentTypeError("failed to parse yaml: %s%s"
+                                         % (e.problem, e.problem_mark))
+    if data is None:
+        raise argparse.ArgumentTypeError("input yaml file is empty")
     return data
 
 
@@ -52,34 +70,64 @@ def read_json_file(path):
 
 
 def read_graphml(path):
+    origpath = path
+    if path == '-':
+        path = sys.stdin
     try:
-        if path == '-':
-            return nx.read_graphml(sys.stdin)
-        else:
-            return nx.read_graphml(path)
-    except:
-        import argparse
+        return nx.read_graphml(path)
+    except IsADirectoryError:
+        raise argparse.ArgumentTypeError("\"%s\" is a directory" % origpath)
+    except PermissionError:
+        raise argparse.ArgumentTypeError("\"%s\" permission denied" % origpath)
+    except FileNotFoundError:
+        raise argparse.ArgumentTypeError("\"%s\" does not exist" % origpath)
+    except xml.etree.ElementTree.ParseError:
         raise argparse.ArgumentTypeError(
-            "Input graph \"%s\" is not in GraphML format" % path)
+            "Input graph \"%s\" is not in GraphML format" % origpath)
 
 
 def read_graph(path):
     origpath = path
+    data = None
+    # we have to slurp all data from stdin as we potentially have to give the
+    # data to read_graphml() *and* to read_dot() if the former failed
     if path == '-':
-        path = sys.stdin
+        data = sys.stdin.read()
     # try reading graphml first because xml.etree.ElementTree will properly
     # fail if it cannot read the input while pygraphviz will parse XML
     # without raising an exception
     # monkey patching the output graph to not loose the input file type
     try:
-        g = nx.read_graphml(path)
+        if data is None:
+            g = nx.read_graphml(path)
+        else:
+            g = nx.read_graphml(io.StringIO(data))
         g.input_file_type = "graphml"
-    except:
+    except IsADirectoryError:
+        raise argparse.ArgumentTypeError("\"%s\" is a directory" % origpath)
+    except PermissionError:
+        raise argparse.ArgumentTypeError("\"%s\" permission denied" % origpath)
+    except FileNotFoundError:
+        raise argparse.ArgumentTypeError("\"%s\" does not exist" % origpath)
+    except xml.etree.ElementTree.ParseError:
         try:
-            g = nx.read_dot(path)
+            if data is None:
+                g = nx.nx_agraph.read_dot(path)
+                # pygraphviz adds the name of the input file as the 'file'
+                # graph attribute if the deprecated "file" argument is used
+                # with pygraphviz.AGraph() as it is done by networkx
+                # https://github.com/pygraphviz/pygraphviz/issues/102
+                del g.graph['graph']['file']
+            else:
+                # we cannot use nx.nx_agraph.read_dot because that uses the
+                # deprecated "file" argument when calling pygraphviz.AGraph()
+                # which does not allow one to pass a file object or a string
+                # with the data
+                import pygraphviz
+                A = pygraphviz.AGraph(string=data)
+                g = nx.nx_agraph.from_agraph(A)
             g.input_file_type = "dot"
         except:
-            import argparse
             raise argparse.ArgumentTypeError(
                 "Input graph \"%s\" is neither in GraphML nor in dot format"
                 % origpath)
@@ -95,7 +143,7 @@ def write_graph(path):
             if g.input_file_type == "graphml":
                 nx.write_graphml(g, sys.stdout.buffer)
             elif g.input_file_type == "dot":
-                nx.write_dot(g, sys.stdout.buffer)
+                nx.nx_agraph.write_dot(g, sys.stdout)
             else:
                 raise Exception("input_file_type attribute must either be dot "
                                 "or graphml")
@@ -104,7 +152,7 @@ def write_graph(path):
             if path.endswith(".xml"):
                 nx.write_graphml(g, path)
             elif path.endswith(".dot"):
-                nx.write_dot(g, path)
+                nx.nx_agraph.write_dot(g, path)
             else:
                 # cannot determine desired output file type from output file
                 # extension, fall back to input file type
@@ -114,7 +162,7 @@ def write_graph(path):
                 if g.input_file_type == "graphml":
                     nx.write_graphml(g, path)
                 elif g.input_file_type == "dot":
-                    nx.write_dot(g, path)
+                    nx.nx_agraph.write_dot(g, path)
                 else:
                     raise Exception("input_file_type attribute must either be "
                                     "dot or graphml")
@@ -130,9 +178,9 @@ def write_graphml(path):
 
 def write_dot(path):
     if path == '-':
-        return lambda g: nx.write_dot(g, sys.stdout)
+        return lambda g: nx.nx_agraph.write_dot(g, sys.stdout)
     else:
-        return lambda g: nx.write_dot(g, path)
+        return lambda g: nx.nx_agraph.write_dot(g, path)
 
 
 class fh_out:
@@ -276,6 +324,7 @@ def graph_remove_droppable(g, reduced_deps):
     droppable_edges = [e for e in g.edges_iter() if is_droppable(e)]
     g.remove_edges_from(droppable_edges)
 
+
 apt_pkg.init()
 
 
@@ -298,6 +347,7 @@ def sort_pkgs(pkg1, pkg2):
         else:
             return cmp(a1, a2)
 
+
 sort_pkgs_key = cmp_to_key(sort_pkgs)
 
 _arch_matches_cache = dict()
@@ -325,8 +375,8 @@ def parse_dose_yaml(yamlin):
     if yamlin.get('output-version') is None:
         raise Exception('missing yaml field output-version')
 
-    if yamlin['output-version'] != '1.0':
-        raise Exception('yaml output-version is unequal 1.0: ' +
+    if yamlin['output-version'] not in ['1.2']:
+        raise Exception('yaml output-version is unsupported: ' +
                         yamlin['output-version'])
 
     if yamlin.get('report') is None:
@@ -349,7 +399,7 @@ def parse_dose_yaml(yamlin):
     return data
 
 
-def vpkgdisj2deb(disj):
+def vpkgdisj2deb(disj, parentarch, nativearch, hostarch, parenttype):
     disjl = disj.split('|')
 
     def vpkg2deb(vpkg):
@@ -362,23 +412,43 @@ def vpkgdisj2deb(disj):
         try:
             name, arch = name.split(':', 1)
         except ValueError:
-            arch = None
+            # if the vpkg does not have an architecture qualification, then
+            # the architecture of the dependee has to be either the hostarch,
+            # the native arch or the parent arch, depending on whether the
+            # depender is a source package, an arch:all package or an arch:any
+            # package
+            if parenttype == "src":
+                arch = hostarch
+            elif parentarch == "all":
+                arch = nativearch
+            else:
+                arch = parentarch
+            arch = parentarch
         if name.startswith('--virtual-'):
             name = name[10:]
         if ver:
             return "%s:%s %s" % (name, arch, ver)
         else:
-            if arch:
-                return "%s:%s" % (name, arch)
-            else:
-                return name
+            return "%s:%s" % (name, arch)
     # make unique, sort and join
     return " | ".join(sorted(list(set([vpkg2deb(vpkg) for vpkg in disjl]))))
 
 
-def get_depchain(depchain):
-    firstpkg = ("", None, vpkgdisj2deb(depchain['depchain'][0]['depends']))
-    chain = [(k['package'], k['version'], vpkgdisj2deb(k['depends']))
+def get_depchain(depchain, parentarch, nativearch, hostarch, parenttype):
+    firstpkg = ("", None, None,
+                vpkgdisj2deb(depchain['depchain'][0]['depends'], parentarch,
+                             nativearch, hostarch, parenttype))
+
+    def get_vpkg(k):
+        if k.get('depends') is None:
+            # if the depchain element does not have a 'depends' field then it
+            # will be the last element of the chain and be in the chain because
+            # it is Essential:yes
+            return "implicit dependency via an Essential:yes package"
+        else:
+            return vpkgdisj2deb(k['depends'], k['architecture'], nativearch,
+                                hostarch, "bin")
+    chain = [(k['package'], k['architecture'], k['version'], get_vpkg(k))
              for k in depchain['depchain'][1:]]
     return [firstpkg] + chain
 
@@ -390,54 +460,73 @@ def parse_dose_yaml_mc(yamlin):
     if yamlin.get('output-version') is None:
         raise Exception('missing yaml field output-version')
 
-    if yamlin['output-version'] != '1.0':
-        raise Exception('yaml output-version is unequal 1.0: ' +
+    if yamlin['output-version'] != '1.2':
+        raise Exception('yaml output-version is unequal 1.2: ' +
                         yamlin['output-version'])
 
     if yamlin.get('report') is None:
         return ({}, {})
 
+    nativearch = yamlin['native-architecture']
+    hostarch = yamlin.get('host-architecture', nativearch)
+
     for p in yamlin['report']:
-        n, v = p['package'], p['version']
+        n, v, a = p['package'], p['version'], p['architecture']
+        t = p.get("type", "bin")
+        if t == "src":
+            n = "src:" + n
         # only add name to avoid duplicates when more than one
         # version exists
         if p['status'] == 'broken':
             for r in p['reasons']:
                 if r.get('missing'):
                     pkg = r['missing']['pkg']
-                    unsatdep = vpkgdisj2deb(pkg['unsat-dependency'])
-                    lastpkg = (pkg['package'], pkg['version'], unsatdep)
+                    unsatdep = vpkgdisj2deb(pkg['unsat-dependency'],
+                                            pkg['architecture'], nativearch,
+                                            hostarch, t)
+                    lastpkg = (pkg['package'], pkg['architecture'],
+                               pkg['version'], unsatdep)
                     c = r['missing'].get('depchains')
                     if c:
-                        depchains = [tuple(get_depchain(depchain) + [lastpkg])
+                        depchains = [tuple(get_depchain(depchain, a,
+                                                        nativearch, hostarch,
+                                                        t) + [lastpkg])
                                      for depchain in c]
                         # normalize the order of depchains
                         depchains = tuple(sorted(set(depchains)))
                     else:
-                        depchains = tuple([(("", None, unsatdep),)])
-                    missing[unsatdep][depchains].add((n, v, None))
+                        depchains = tuple([(("", None, None, unsatdep),)])
+                    missing[unsatdep][depchains].add((n, a, v, None))
                 if r.get('conflict'):
                     conf1 = r['conflict']['pkg1']
                     conf2 = r['conflict']['pkg2']
-                    unsatconf = vpkgdisj2deb(conf1['unsat-conflict'])
-                    lastpkg1 = (conf1['package'], conf1['version'], unsatconf)
-                    lastpkg2 = (conf2['package'], conf2['version'], None)
+                    unsatconf = vpkgdisj2deb(conf1['unsat-conflict'],
+                                             conf1['architecture'], nativearch,
+                                             hostarch, t)
+                    lastpkg1 = (conf1['package'], conf1['architecture'],
+                                conf1['version'], unsatconf)
+                    lastpkg2 = (conf2['package'], conf2['architecture'],
+                                conf2['version'], None)
                     c1 = r['conflict'].get('depchain1')
                     if c1:
-                        depchain1 = [tuple(get_depchain(depchain) + [lastpkg1])
+                        depchain1 = [tuple(get_depchain(depchain, a,
+                                                        nativearch, hostarch,
+                                                        t) + [lastpkg1])
                                      for depchain in c1]
                         depchain1 = tuple(sorted(set(depchain1)))
                     else:
-                        depchain1 = tuple([(("", None, unsatconf),)])
+                        depchain1 = tuple([(("", None, None, unsatconf),)])
                     c2 = r['conflict'].get('depchain2')
                     if c2:
-                        depchain2 = [tuple(get_depchain(depchain) + [lastpkg2])
+                        depchain2 = [tuple(get_depchain(depchain, a,
+                                                        nativearch, hostarch,
+                                                        t) + [lastpkg2])
                                      for depchain in c2]
                         depchain2 = tuple(sorted(set(depchain2)))
                     else:
-                        depchain2 = tuple([(("", None, None),)])
+                        depchain2 = tuple([(("", None, None, None),)])
                     depchains = tuple([depchain1, depchain2])
-                    conflict[unsatconf][depchains].add((n, v, None))
+                    conflict[unsatconf][depchains].add((n, a, v, None))
     return (missing, conflict)
 
 
